/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.commenting.CommentForm");

dojo.require("rmps.core.util.StringUtils");
dojo.require("rmps.core.commenting.Comment");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dijit.form.Form");
dojo.require("dijit.form.Button");

dojo.requireLocalization("rmps.ui.commenting", "CommentFormMessages");

var BODY_TO_SUBJECT_LENGTH = 40;
var StringUtils = rmps.core.util.StringUtils;

dojo.declare("rmps.ui.commenting.CommentForm", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.ui", "commenting/templates/CommentForm.html"),
	widgetsInTemplate: true,
	
	messages: dojo.i18n.getLocalization("rmps.ui.commenting", "CommentFormMessages"),
	
	// attach points
	_subjectInput: null,
	_bodyInput: null,
	_cancelButton: null,
	_controlBar: null,
	_subjectContainer: null,
	
	// mixin properties
	commentingWidget: null,
	comment: null,
	showSubjectField: true,
	
	onPost: null,
	onSave: null,
	onCancel: null,
	
	// private properties
	_posted: false,
	_isDerivedSubject: false,
	_isValid: false,
	
	resize: function(size) {
		if (size && size.h != null) {
			var bodyInputMb = dojo.marginBox(this._bodyInput);
			var mb = dojo.marginBox(this.domNode);
			var newHeight = Math.max(size.h - (mb.h - bodyInputMb.h), dojo.marginBox(this._subjectInput).h);
			dojo.marginBox(this._bodyInput, {h: newHeight });
		}
	},
	
	update: function(comment) {
		this._isDerivedSubject = false;
		dojo.removeClass(this._subjectInput, "derivedInput");
		this._enable(true);
		this._posted = false;
		
		this.comment = comment;
		
		var body = this.comment.getBody(rmps.core.commenting.BodyType.PLAIN_TEXT.value);
		
		if (this.showSubjectField) {
			this._subjectContainer.style.display = "";
		} else {
			this._subjectContainer.style.display = "none";
		}
		
		this._subjectInput.value = this.comment.subject ? this.comment.subject : "";
		this._bodyInput.value = (body && body.contents ? body.contents : "");
		this._validateInput();
	},
	
	setFocus: function() {
		if (!this.showSubjectField || (this.comment.subject && this.comment.subject.length > 0)) {
			this._bodyInput.focus();
		} else {
			this._subjectInput.focus();
		}
	},
	
	/**
	 * <p>
	 * Validates the commenting form, dispatching an error and returning <code>FALSE</code> if 
	 * there is a problem.  
	 * </p>
	 * 
	 * @return <code>TRUE</code> if the form is valid, <code>FALSE</code> otherwise 
	 */
	validate: function() {
		if(this._isValid)
			return true;
		
		// #_validateInput() is called for every keypress, therefore we cannot dispatch the error there.
		// Instead we must create the error when someone tries to do something requiring a valid input form.
		rmps.core.error.create(this.messages.validate_commentSubject, "", this.messages.validateBeforeSave).dispatch();
		return false;
	},
	
	_validateInput: function() {
		if (!this._posted) {
			if (dojo.trim(this._subjectInput.value).length == 0) {
				this._isValid = false;
			} else {
				this._isValid = true;
			}
		}
	},
	
	postFailed: function() {
		this._posted = false;
		this._cancelButton.disabled = false;
	},
	
	_enable: function(enabled) {
		this._subjectInput.disabled = !enabled;
		this._bodyInput.disabled = !enabled;
		this._cancelButton.disabled = !enabled;
	},
	
	_onSubjectKeyPress: function(e) {
		this._isDerivedSubject = this._subjectInput.value.length === 0;
		this._validateInput();
	},
	
	_onBodyKeyPress: function(e) {
		this._computeDerivedSubject();
		this._validateInput();
	},
	
	_computeDerivedSubject: function() {
		if (this._subjectInput.value.length === 0 || this._isDerivedSubject) {
			var bodySubject = dojo.trim(this._bodyInput.value);
			var newLineIdx = bodySubject.indexOf("\n");
			if (newLineIdx != -1) {
				bodySubject = dojo.trim(bodySubject.substring(0, newLineIdx));
			}
			this._subjectInput.value = StringUtils.truncate(bodySubject, BODY_TO_SUBJECT_LENGTH, null, true, true);
			this._isDerivedSubject = true;
			dojo.addClass(this._subjectInput, "derivedInput");
		} else {
			this._isDerivedSubject = false;
			dojo.removeClass(this._subjectInput, "derivedInput");
		}
	},
	
	_onSubjectFocus: function(e) {
		if (this._isDerivedSubject) {
			dojo.removeClass(this._subjectInput, "derivedInput");
		}
	},
	
	_onSubjectBlur: function(e) {
		this._computeDerivedSubject();
		this._validateInput();
	},
	
	// post
	_onPost: function(e) {
		this.post();
	},
	
	post: function() {
		if (this._posted || !this._isValid) {
			return null;
		}
		this._posted = true;
		this._enable(false);

		this.comment.subject = dojo.trim(this._subjectInput.value);
		this.comment.addBody({
			type: rmps.core.commenting.BodyType.PLAIN_TEXT.value,
			contents: this._bodyInput.value.replace(/^(\s*?\n)*/, '').replace(/\s\s*$/, '')
		});
		if (this.onPost) {
			return this.onPost(this.comment);
		}
		return null;
	},
	
	// cancel
	_onCancel: function(e) {
		this.cancel();
	},
		
	cancel: function() {
		if (this.onCancel) {
			this.onCancel(this.comment);
		}
	},
	
	_onSave: function(e) {
		if (this.onSave) {
			this.onSave(this.comment);
		}		
	}
	
});

})();