/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.common.internal.widgets.FrameContainer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
(function(){

 /*
 * It is used to wrap OSLC pickers (creation, selections ,etc) in an iFrame widget. For instance:
 * 
 *          var frameContainerArgs = {
 *               onData: dojo.hitch(this, this.yourOnDataHandling),	//when closing the iFrame it will be invoked	
 *               protocol: this.protocol	//The current window protocol. For instance: "#oslc-windowName-1.0"
 *           };
 *           this.dialogContent = new rmpw.common.internal.widgets.FrameContainer(frameContainerArgs);
 *            this.dialogContent.setFrameSrc(frameSrc);	//The OSLC picker url
 *            var floatingPaneParams = {
 *               primaryTitle: this.dialogTitle,
 *               secondaryTitle: this.secondaryTitle,
 *               width: width,
 *               height: heigth,
 *               onClose: this.onClose,	//onClose will call for onData
 *               contentNode: this.dialogContent.domNode
 *           };
 * 
 *  
 */
dojo.declare("rmpw.common.internal.widgets.FrameContainer", [dijit._Widget, dijit._Templated], {

    frame: null,
    
    constructor: function(args){
        if (!dojo.isFunction(args.onData)) 
            throw new Error("Missing 'onData'");
    },
    
    templateString: '<div dojoAttachPoint="_iframeDiv" class="com-ibm-team-foundation-crossdomain-web-IFrameContainer" style="width: 100%; height: 100%;"> </div>',
    
    postCreate: function(){
        var doc = dojo.doc;
        var self = this;
        var returnLocation = new dojo._Url(window.location, dojo.moduleUrl("dojo", "resources/blank.html")).toString();
        
        this.frame = doc.createElement(dojo.isIE <9 ? '<iframe name="' + returnLocation + '" onload="window.frameLoadCallback()">' : 'iframe');
        this.frame.name = returnLocation;
        dojo.style(this.frame, 'width', '100%');
        dojo.style(this.frame, 'height', '99%');
        this.frame.frameBorder = '0';
        this.frame.marginWidth = '0';
        this.frame.marginHeight = '0';
        this.frame.scrolling = 'auto';
        
        var state = 0;
        this.frame.onload = window.frameLoadCallback = function(){
            if (!state) {
                try {
                    var data = self.frame.contentWindow.name;
                } 
                catch (e) {
                    return;
                };
                if (typeof data == 'string') {
                    if (data != returnLocation) {
                        state = 1;
                        self.onData(data);
                    }
                }
            }
        };
        this._iframeDiv.appendChild(this.frame);
    },
    
    setFrameSrc: function(remoteLocation){
        var frameSrc = remoteLocation;
        if (this.protocol) {
            frameSrc = remoteLocation + this.protocol;
        }
        this.frame.src = frameSrc;
    }
});
    
})();
