/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide('com.ibm.team.vvc.shared.web.ui.util.LinkToWorkItemUtil');

dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
dojo.require("com.ibm.team.vvc.shared.oslc.web.ui.dialogs.AddCALMLinksDialog");
dojo.require("com.ibm.team.vvc.shared.oslc.web.ui.actions.ShowLinkedElementsDialogAction");

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.util", "LinkToWorkItemUtilMessages");

var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
var Messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.util", "LinkToWorkItemUtilMessages");

/**
 * This utility class is used to support links to work items.
 */
com.ibm.team.vvc.shared.web.ui.util.LinkToWorkItemUtil = {
		
	showLinkedWorkItemsAction: function() {
		
		com.ibm.team.vvc.shared.oslc.web.ui.actions.ShowLinkedElementsDialogAction.showDialog(ContextUtils.getCurrentContext().uri,
			function() {
				return { url : net.jazz.ajax.contextRoot() + '/vvcshared/proxy', headers : {
					vvc_servicePath : "/changesetLinks?changesetConfiguration=" + ContextUtils.getCurrentContext().uri +
					"&consumerUrl=" + net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL
				}};
			},
			function(targetUrl) {
				return { url : net.jazz.ajax.contextRoot() + '/vvcshared/proxy', headers : {
					vvc_servicePath : "/changesetLinks?changesetConfiguration=" + ContextUtils.getCurrentContext().uri +
					"&predicate=http%3A%2F%2Fjazz.net%2Fns%2Fvvc%2Flink%2FtracksChanges%2Frtc&backlinkUrl=http%3A%2F%2Fopen-services.net%2Fns%2Fcm%23tracksChangeSet&target=" +
					targetUrl + "&consumerUrl=" + net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL
				}};
			}
		);
	},
	
	//this method is intended for consumers, which typically are in a changeset configuration
	linkToWorkItemAction: function(consumerMode) {
		
		var context = ContextUtils.getCurrentContext();
		//otherwise, we could throw an error (although this should never really happen because the caller checks in isApplicable())
		if (context.isChangeset()) {
			var subjectUri = ContextUtils.getCurrentContext().uri;
			
			var dialog = com.ibm.team.vvc.shared.oslc.web.ui.dialogs.AddCALMLinksDialog;
			var params = {
				consumerMode: true,
	            dialogTitle: Messages.LinkToWorkItemDialogTitle,
	            //no need to add dialogSecondaryTitle or item
				onOk: function onOk(item){
					var targetUrl = item.url;
					if (targetUrl) {
						var args = {
							headers: {
								"Content-Type": "application/x-oslc-common-link+xml",
								"Accept": "application/atom+json"
							},

							//http://jazz.net/ns/vvc/link/tracksChanges/rtc
							//http://open-services.net/ns/cm#tracksChangeSet
							//encoded to		
							//http%3A%2F%2Fjazz.net%2Fns%2Fvvc%2Flink%2FtracksChanges%2Frtc
							//http%3A%2F%2Fopen-services.net%2Fns%2Fcm%23tracksChangeSet

							url: net.jazz.ajax.contextRoot() + '/vvcshared/proxy',
							headers : {
								vvc_servicePath : "/changesetLinks?changesetConfiguration=" + subjectUri +
								"&predicate=http%3A%2F%2Fjazz.net%2Fns%2Fvvc%2Flink%2FtracksChanges%2Frtc&backlinkPredicate=http%3A%2F%2Fopen-services.net%2Fns%2Fcm%23tracksChangeSet&target=" +
								targetUrl + "&consumerUrl=" + net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "&backlinkUrl=http://open-services.net/ns/cm%23tracksChangeSet"
							},											
//							load: dojo.hitch(this, function(response, ioArgs) {
//								//possibly do a callback here
//							}),
							handleAs: "json",
							hasBody: false
						};
						var deferred = jazz.client.xhrPost(args);
						deferred.addErrback(this, dojo.hitch(this, function(error) {
							alert(Messages.errorAddingLink);
						}));
						return deferred;
						 
					}
				},
	            resourceUri: subjectUri,
	            acceptableLinkTypes: ["http://open-services.net/ns/cm#"]
			};
			dialog = new dialog(params);
			
		}
	},

	//this method is intended for vvc directly, so there is direct access to the changeset
	vvcLinkToWorkItemAction: function(subjectUri, deferred) {
		
		var context = {
			deferred: deferred
		};
		
		var dialog = com.ibm.team.vvc.shared.oslc.web.ui.dialogs.AddCALMLinksDialog;
		var params = {
            dialogTitle: Messages.LinkToWorkItemDialogTitle,
            //no need to add dialogSecondaryTitle or item
			onOk: dojo.hitch(context, function onOk(item){
				var targetUrl = item.url;
				if (targetUrl) {
					var args = {
						headers: {
							"Content-Type": "application/x-oslc-common-link+xml",
							"Accept": "application/atom+json"
						},

						//http://jazz.net/ns/vvc/link/tracksChanges/rtc
						//http://open-services.net/ns/cm#tracksChangeSet
						//encoded to		
						//http%3A%2F%2Fjazz.net%2Fns%2Fvvc%2Flink%2FtracksChanges%2Frtc
						//http%3A%2F%2Fopen-services.net%2Fns%2Fcm%23tracksChangeSet

						url: net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/changesetLinks?changeset=" + subjectUri +
							"&predicate=http%3A%2F%2Fjazz.net%2Fns%2Fvvc%2Flink%2FtracksChanges%2Frtc&backlinkPredicate=http%3A%2F%2Fopen-services.net%2Fns%2Fcm%23tracksChangeSet&target=" +
							targetUrl + "&consumerUrl=" + net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "&backlinkUrl=http://open-services.net/ns/cm%23tracksChangeSet",
//							load: dojo.hitch(this, function(response, ioArgs) {
//								//possibly do a callback here
//							}),
						handleAs: "json",
						hasBody: false
					};
					var xhrPostDeferred = jazz.client.xhrPost(args);
					var context = {
						deferred: this.deferred
					}
					xhrPostDeferred.addCallback(context, dojo.hitch(this, function(result) {
						if (this.deferred) {
							this.deferred.resolve(result);
						}
					}));
					xhrPostDeferred.addErrback(context, dojo.hitch(this, function(error) {
						alert(Messages.errorAddingLink);
						if (this.deferred) {
							this.deferred.reject(error);
						}
					}));
					 
				}
			}),
            resourceUri: subjectUri,
            acceptableLinkTypes: ["http://open-services.net/ns/cm#"]
		};
		dialog = new dialog(params);
	}	
};

})();			